
package ft.html;
import junit.framework.*;
import fit.Parse;
import java.net.URL;
import java.util.*;
import java.text.ParseException;
import com.gargoylesoftware.htmlunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import com.gargoylesoftware.htmlunit.test.FakeWebConnection;

public class HtmlFixtureTest extends TestCase
{
	private Parse parse;
	private HtmlPage htmlPage;
	private HtmlFixture fixture;
	private Parse idCell;
	private Parse typeCell;
	private Parse nameCell;
	private Parse valueCell;
	private Parse titleCell;
	private Parse textCell;

	public HtmlFixtureTest(String name)
	{
		super(name);
	}

	private static String html =
	   "<html>" +
	   " <head>" +
	   "   <title>This is the Title</title>" +
		"   <script>" +
		"     function processClick()" +
		"     {" +
		"       document.f.t.value = \"button clicked\";" +
		"     }" +
		"   </script>" +
	   " </head>" +
	   "<body id=\"bodyTag\">" +
	   " This is the body" +
		"<form name=\"f\">" +
		"  <input type=\"button\" name=\"b\" value=\"Click\" onclick=\"processClick()\">" +
		"  <input type=\"text\" cols=\"30\" name=\"t\" value=\"initial\">" +
		"</form>" +
	   "</body>" +
	   "</html>";


	public void setUp() throws Exception
	{
		htmlPage = makeHtmlPage(html);
		fixture = new HtmlFixture();
	}

	public void testRootElement() throws Exception
	{
		prepareTest(null);
		assertEquals("testElement", fixture.htmlSource);
		assertSame(htmlPage, fixture.element);
	}

	public void testIsValidUrl() throws Exception
	{
		assertTrue(HtmlFixture.isUrlValid("http://localhost/hi"));
		assertTrue(HtmlFixture.isUrlValid("ftp://localhost/hi"));
		assertFalse(HtmlFixture.isUrlValid("blah"));
	}

	public void testElementsRow() throws Exception
	{
		prepareTest("<td>Elements</td><td>2</td>");
		Parse greenCell = parse.parts.more.more.parts.more;
		assertEquals("2", greenCell.text());
		assertRight(greenCell);
	}

	private void prepareElementTest(String cells) throws ParseException
	{
		prepareTest(cells);
		titleCell = parse.parts.more.more.parts;
		idCell = titleCell.more;
		typeCell = idCell.more;
		nameCell = typeCell.more;
	}

	public void testElementRowUsingIndex_AllPass() throws Exception
	{
		prepareElementTest("<td>Element</td><td>2</td><td>body</td><td>bodyElement</td>");
		assertRight(idCell);
		assertRight(typeCell);
		assertRight(nameCell);

		HtmlElement body = (HtmlElement)htmlPage.getChildElements().get(1);
		assertSame(body, HtmlFixture.retrieve("bodyElement"));
	}

	public void testElementRowUsingIndex_FailingType() throws Exception
	{
		prepareElementTest("<td>Element</td><td>2</td><td>bodyBlah</td><td>bodyElement</td>");
		assertRight(idCell);
		assertWrong(typeCell);
		assertRight(nameCell);
	}

	public void testElementRowWithBadIndex() throws Exception
	{
		prepareElementTest("<td>Element</td><td>5001</td><td>blah</td><td>blah</td>");
		assertWrong(idCell);
		assertIgnored(typeCell);
		assertIgnored(nameCell);
	}

	public void testElementRowWithNoNameCell() throws Exception
	{
		try
		{
			prepareElementTest("<td>Element</td><td>1</td><td>head</td>");
		}
		catch(Exception e)
		{
			fail("no exception should be thrown when the nameCell is not present");
		}
	}

	public void testElementRowUsingId() throws Exception
	{
		prepareElementTest("<td>Element</td><td>bodyTag</td><td>body</td><td>bodyElement</td>");
		assertRight(idCell);
		assertRight(typeCell);
		assertRight(nameCell);

		HtmlElement body = (HtmlElement)htmlPage.getChildElements().get(1);
		assertSame(body, HtmlFixture.retrieve("bodyElement"));
	}

	public void testElementRowWithMissingId() throws Exception
	{
		prepareElementTest("<td>Element</td><td>blah</td><td>body</td><td>bodyElement</td>");
		assertWrong(idCell);
		assertIgnored(typeCell);
		assertIgnored(nameCell);
	}

	private void prepareAttributeTest(String cells) throws ParseException
	{
		HtmlElement body = (HtmlElement)htmlPage.getChildElements().get(1);
		HtmlFixture.store("testElement", body);
		parse = new Parse(makeHtmlFixtureTable("testElement", cells));
		fixture.doTable(parse);
		titleCell = parse.parts.more.more.parts;
		idCell = titleCell.more;
		valueCell = idCell.more;
	}

	public void testAttributeRow_AllPassing() throws Exception
	{
		prepareAttributeTest("<td>Attribute</td><td>id</td><td>bodyTag</td>");
		assertRight(idCell);
		assertRight(valueCell);
	}

	public void testAttributeRow_FailingValue() throws Exception
	{
		prepareAttributeTest("<td>Attribute</td><td>id</td><td>blah</td>");
		assertRight(idCell);
		assertWrong(valueCell);
	}

	public void testAttributeRow_MissingAttribute() throws Exception
	{
		prepareAttributeTest("<td>Attribute</td><td>blah</td><td>blah</td>");
		assertWrong(idCell);
		assertIgnored(valueCell);
	}

	public void testAttributeRow_NoValueCell() throws Exception
	{
		try
		{
			prepareAttributeTest("<td>Attribute</td><td>id</td>");
			assertRight(idCell);
		}
		catch(Exception e)
		{
			fail("should not throw exception when valueCell is omitted");
		}
	}

	public void testBadRowName() throws Exception
	{
		prepareElementTest("<td>Blah</td><td>blah</td><td>blah</td><td>blah</td>");
		assertWrong(titleCell);
		assertIgnored(idCell);
		assertIgnored(typeCell);
		assertIgnored(nameCell);
	}

	public void testMissingElement() throws Exception
	{
		parse = new Parse(makeHtmlFixtureTable("testElement", "<td>abc</td>"));
		HtmlFixture.store("testElement", null);
		fixture.doTable(parse);

		Parse sourceCell = parse.parts.more.parts;
		assertWrong(sourceCell);
		Parse nextRow = parse.parts.more.more.parts;
		assertIgnored(nextRow);
	}

	public void testMissingHtmlSourceRow() throws Exception
	{
		parse = new Parse("<table><tr><td>HtmlFixture</td></tr></table>");
		fixture.doTable(parse);

		Parse fixtureCell = parse.parts.parts;
		assertIgnored(fixtureCell);
	}

	private void prepareTextTest(String cells) throws ParseException
	{
		HtmlElement head = (HtmlElement)htmlPage.getChildElements().get(0);
		HtmlElement title = (HtmlElement)head.getChildElements().get(0);
		HtmlFixture.store("testElement", title);
		parse = new Parse(makeHtmlFixtureTable("testElement", cells));
		fixture.doTable(parse);
		titleCell = parse.parts.more.more.parts;
		textCell = titleCell.more;
	}

	public void testTextRow_Pass() throws Exception
	{
		prepareTextTest("<td>Text</td><td>This is the Title</td>");
		assertRight(textCell);
	}

	public void testTextRow_Fail() throws Exception
	{
		prepareTextTest("<td>Text</td><td>blah</td>");
		assertWrong(textCell);
	}

	public void testHasTextRow_Pass() throws Exception
	{
		prepareTextTest("<td>Has Text</td><td>the Title</td>");
		assertRight(textCell);
	}

	public void testHasTextRow_Fail() throws Exception
	{
		prepareTextTest("<td>Has Text</td><td>blah</td>");
		assertWrong(textCell);
	}

	public void testMatchesTextRow_Pass() throws Exception
	{
		prepareTextTest("<td>Matches Text</td><td>This.*Title</td>");
		assertRight(textCell);
	}

	public void testMatchesTextRow_Fail() throws Exception
	{
		prepareTextTest("<td>Matches Text</td><td>This.Title</td>");
		assertWrong(textCell);
	}

	public void testExecuteJavascript() throws Exception
	{
		HtmlForm form = htmlPage.getFormByName("f");
		HtmlInput textElement   = form.getInputByName("t");
		HtmlInput buttonElement = form.getInputByName("b");

		fixture.htmlPage = htmlPage;
		fixture.store("textElement", textElement);
		fixture.store("buttonElement", buttonElement);

		Parse parse = new Parse(makeHtmlFixtureTable("textElement", "<td>Text</td><td>initial</td>"));
		fixture.doTable(parse);
		assertRight(parse.parts.more.more.parts.more);

		parse = new Parse(makeHtmlFixtureTable("textElement", "<td>Attribute</td><td>value</td><td>initial</td>"));
		fixture.doTable(parse);
		assertRight(parse.parts.more.more.parts.more);

		parse = new Parse(makeHtmlFixtureTable("buttonElement", "<td>Execute</td><td>OnClick</td>"));
		fixture.doTable(parse);

		parse = new Parse(makeHtmlFixtureTable("textElement", "<td>Text</td><td>button clicked</td>"));
		fixture.doTable(parse);
		assertRight(parse.parts.more.more.parts.more);
	}

	public void testInabilityToParseXml() throws Exception
	{
		makeSampleHtmlWithXml();
		htmlPage = makeHtmlPage(html);
		HtmlElement body = (HtmlElement)htmlPage.getChildElements().get(1);
		List bodyElements = body.getChildElements();
		assertEquals(1, bodyElements.size());
	}

	private void makeSampleHtmlWithXml()
	{
		html =
			"<html>" +
			" <head>" +
			"   <title>This is the Title</title>" +
			" </head>" +
			"<body id=\"bodyTag\">" +
			" This is the body" +
		   "<input name=inputName1 type=text value=\"some text\">" +
			"<?XML:NAMESPACE PREFIX=TVNS />" +
			"<?IMPORT NAMESPACE=TVNS IMPLEMENTATION=\"treeview.htc\" />" +
			"<tvns:treeview id=\"_ctl1_Projecttree_TreeView1\" defaultStyle=\"font-family:Verdana, Helvetica, sans-serif;font-size:8pt;font-weight:normal;\" childType=\"Folder\" selectedNodeIndex=\"0.0\" HelperID=\"___ctl1_Projecttree_TreeView1_State__\" systemImagesPath=\"/webctrl_client/1_0/treeimages/\" Focused=\"true\" __scrollTop=\"0\" __scrollLeft=\"0\" __parentTop=\"0\" __parentLeft=\"0\" onexpand=\"java: if (this.clickedNodeIndex != null) this.queueEvent('onexpand', this.clickedNodeIndex)\" oncollapse=\"javascript: if (this.clickedNodeIndex != null) this.queueEvent('oncollapse', this.clickedNodeIndex)\" oncheck=\"javascript: if (this.clickedNodeIndex != null) this.queueEvent('oncheck', this.clickedNodeIndex)\" onselectedindexchange=\"javascript: if (event.oldTreeNodeIndex != event.newTreeNodeIndex) this.queueEvent('onselectedindexchange', event.oldTreeNodeIndex + ',' + event.newTreeNodeIndex)\" onfirequeuedevents=\"javascript: window.setTimeout('__doPostBack(\'_ctl1$Projecttree$TreeView1\',\'\')', 0, 'JavaScript')\" style=\"color:Black;\" />" +
			"</body>" +
			"</html>";
	}

	private void assertRight(Parse parse)
	{
		String tag = parse.tag;
		assertTrue("The cell is not green " + tag, tag.matches(".*#cfffcf.*"));
	}

	private void assertWrong(Parse parse)
	{
		String tag = parse.tag;
		assertTrue("The cell is not red " + tag, tag.matches(".*#ffcfcf.*"));
	}

	private void assertIgnored(Parse parse)
	{
		String tag = parse.tag;
		assertTrue("The cell is not grey " + tag, tag.matches(".*#efefef.*"));
	}

	public static HtmlPage makeHtmlPage(String html) throws Exception
	{
		WebClient client = new WebClient();
		FakeWebConnection webConnection = new FakeWebConnection(client);
		webConnection.setResponse(new URL("http://first"), html, 200, "OK", "text/html", Collections.EMPTY_LIST);
		client.setWebConnection(webConnection);
		HtmlPage htmlPage = (HtmlPage)client.getPage(new URL("http://first"), SubmitMethod.GET, Collections.EMPTY_LIST);

		return htmlPage;
	}

	private String makeHtmlFixtureTable(String elementName, String cells)
	{
		StringBuffer table = new StringBuffer("<table>\n");
		table.append(" <tr>\n");
		table.append("   <td>HtmlFixture</td>\n");
		table.append(" </tr>\n");
		table.append(" <tr>\n");
		table.append("   <td>" + elementName + "</td>\n");
		table.append(" </tr>\n");
		if(cells != null)
		{
			table.append(" <tr>\n");
			table.append(cells).append("\n");
			table.append(" </tr>\n");
		}
		table.append("</table>\n");

		return table.toString();
	}

	private void prepareTest(String row) throws ParseException
	{
		parse = new Parse(makeHtmlFixtureTable("testElement", row));
		HtmlFixture.store("testElement", htmlPage);
		fixture.doTable(parse);
	}
}